@@ -120,7 +120,8 @@ class HTMAttention(nn.Module):
         self,
         queries,
         memories,
-        mask = None
+        mask = None,
+        chunk_attn_mask = None
     ):
         dim, query_len, mem_chunk_size, topk_mems, scale, eps = self.dim, queries.shape[1], self.mem_chunk_size, self.topk_mems, self.scale, self.eps
 
@@ -153,13 +154,16 @@ class HTMAttention(nn.Module):
         # do a single head attention over summary keys
 
         sim = einsum('b i d, b j d -> b i j', summary_queries, summary_keys) * scale
+        mask_value = -torch.finfo(sim.dtype).max
 
         if exists(mask):
             chunk_mask = mask.any(dim = 2)
             chunk_mask = rearrange(chunk_mask, 'b j -> b () j')
-            mask_value = -torch.finfo(sim.dtype).max
             sim = sim.masked_fill(~chunk_mask, mask_value)
 
+        if exists(chunk_attn_mask):
+            sim = sim.masked_fill(~chunk_attn_mask, mask_value)
+
         topk_logits, topk_indices = sim.topk(k = topk_mems, dim = -1)
         weights = topk_logits.softmax(dim = -1)
 
@@ -170,7 +174,7 @@ class HTMAttention(nn.Module):
         # select the topk memories
 
         memories = repeat(memories, 'b m j d -> b m i j d', i = query_len)
-        mem_topk_indices = repeat(topk_indices, 'b i m -> b m i j d', j = mem_chunk_size, d = memories.shape[-1])
+        mem_topk_indices = repeat(topk_indices, 'b i m -> b m i j d', j = mem_chunk_size, d = dim)
         selected_memories = memories.gather(1, mem_topk_indices)
 
         # positional encoding
@@ -200,3 +204,20 @@ class HTMAttention(nn.Module):
         weighted_output = within_mem_output * rearrange(weights, 'b i m -> b m i ()')
         output = weighted_output.sum(dim = 1)
         return output
+
+# HTM Block
+
+class HTMBlock(nn.Module):
+    def __init__(self, dim, **kwargs):
+        super().__init__()
+        self.norm = nn.LayerNorm(dim)
+        self.attn = HTMAttention(dim = dim, **kwargs)
+    def forward(
+        self,
+        queries,
+        memories,
+        **kwargs
+    ):
+        queries = self.norm(queries)
+        out = self.attn(queries, memories, **kwargs) + queries
+        return out
